import os
from flask import Flask

# https://flask.palletsprojects.com/en/1.1.x/tutorial/factory/
def create_app(test_config=None):
    # create and configure the app
    app = Flask(__name__, instance_relative_config=True)

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile("config.py", silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    # import routes
    # import commands
    # must be done after db initialization
    with app.app_context():
        from .routes import print_route
        from .services import print_service

        # register routes
        app.register_blueprint(print_route.bp)
        # register services
        app.register_blueprint(print_service.bp)

        return app
