from flask import Blueprint, request, current_app
import paho.mqtt.publish as publish
from escpos.printer import Network
from json import dumps
from flask_cors import CORS
from datetime import datetime, timezone
import uuid
import ipaddress

from ..models.ticket_model import Ticket

bp = Blueprint("api", __name__, url_prefix="/api/print")
CORS(bp)


@bp.route("/submit", methods=["POST"])
def submit():
    """
    A route to handle print request.
    """

    response = {}

    req = request.json

    try:
        for item in req:
            if not "imprimanteId" in item:
                raise Exception("imprimanteId field required but not found!")
            if not "imprimanteIp" in item:
                raise Exception("imprimanteIp field required but not found!")
            if not "caisseIp" in item:
                raise Exception("caisseIp field required but not found!")

            # Check IP address
            ipaddress.ip_address(item["imprimanteIp"])

            topic = "/printserver/printer%d" % (item["imprimanteId"])

            current_app.logger.info(topic)
            publish.single(
                topic,
                payload=dumps(item),
                qos=2,
                hostname=current_app.config["MQTT_BROKER_URL"],
                port=current_app.config["MQTT_BROKER_PORT"],
                client_id="api",
            )

        response["status"] = 200

    except Exception as ex:
        response["status"] = 400
        response["error"] = str(ex)
        current_app.logger.error(ex)

    # Sending response
    return response, response["status"]


@bp.route("/status", methods=["POST"])
def status():
    """
    A route to handle status request.
    """

    response = {}

    if not isinstance(request.json, list):
        response["status"] = 400
        response["error"] = "Please provide a valide list of IP addresses !"
        return response, response["status"]

    req = request.json
    response["printers"] = []
    try:
        # todo: input must be array/list
        for printer_ip in req:
            current_app.logger.info("Checking printer " + str(printer_ip) + " ...")
            response["printers"].append(check_printer(printer_ip))

        response["status"] = 200

    except Exception as ex:
        response["status"] = 400
        response["error"] = str(ex)
        current_app.logger.error(ex)

    # Sending response
    return response, response["status"]


def check_printer(printer_ip):
    try:
        # Check correct Ip Address
        ipaddress.ip_address(printer_ip)
        # Connect de printer
        printer = Network(
            printer_ip,
            timeout=current_app.config["PRINTER_TIMEOUT"],
        )

        return {"ip": printer_ip, "online": True, "error": ""}

    except ipaddress.AddressValueError as ex:
        current_app.logger.error(ex)
        # return {"ip": printer_ip, "online": False, "error": "Invalide IP address!"}
        return {"ip": printer_ip, "online": False, "error": str(ex)}

    except Exception as ex:
        current_app.logger.error(ex)
        # return {"ip": printer_ip, "online": False, "error": "Printer unreachable!"}
        return {"ip": printer_ip, "online": False, "error": str(ex)}
