from flask import Blueprint, request, current_app
import paho.mqtt.client as mqtt
import concurrent.futures
import threading
import click
from time import sleep
from json import loads
from json import dumps

bp = Blueprint("threads", __name__)


def do_work(app, printerId):
    def handle_mqtt_connect(mqttc, userdata, flags, rc):
        pass
        # current_app.logger.info("rc: " + str(rc))

    def handle_mqtt_message(mosq, userdata, msg):
        # print(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))
        send_to_printer(msg.topic, "")

    def send_to_printer(printer, content):
        app.logger.info(f"{printer}: start")
        for i in range(3):
            app.logger.info(f"{printer}: line {i+1}")
            sleep(1)
        app.logger.info(f"{printer}: done")
        pass

    mqttc = mqtt.Client("threads_" + str(printerId))

    # Add message callbacks that will only trigger on a specific subscription match.
    mqttc.on_connect = handle_mqtt_connect
    mqttc.on_message = handle_mqtt_message

    app.logger.info("Mqtt Client connecting ...")
    mqttc.connect(
        app.config.get("MQTT_BROKER_URL"),
        app.config.get("MQTT_BROKER_PORT"),
        app.config.get("MQTT_KEEPALIVE"),
    )
    app.logger.info("Mqtt Client connected!")

    # To move to on_connect to be able to subscribe on each re-connection
    mqttc.subscribe("/printserver/printer" + str(printerId) + "/#", 2)

    mqttc.loop_forever()


@bp.cli.command("print")
def print_command():
    """
    Reads from Mqtt topic, processes data and sends it to printer.

    $ flask services print
    """

    # Extracting the app and pass it to the function over the thread
    app = current_app._get_current_object()
    with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
        threads = []
        for printerId in range(5):
            current_app.logger.info("Creating thread for printer %d" % (printerId))
            threads.append(executor.submit(do_work, app=app, printerId=printerId))
        for thread in concurrent.futures.as_completed(threads):
            try:
                print(thread.result())
            except Exception as ex:
                current_app.logger.error(ex)
                # raise ex
